@@ -4,15 +4,21 @@ use version; $VERSION = qv('0.0.6');
 
 use warnings;
 use strict;
-use Carp;
+use Carp qw( carp cluck confess croak );
 use DBI;
 use Readonly;
-use Time::HiRes qw( usleep );
+use Time::HiRes qw{ usleep };
 use English '-no_match_vars';
+use Switch 'Perl6';
+use Data::Types qw{ :is };
 use YAML;
 use Want;
+use Scalar::Util;
+use Exception::Class;
+#use List::Util qw{ first };
 
 use Object::POOF::X;
+use Object::POOF::Tools qw( :data );
 use Object::POOF::Constants;
 use Config::Std;
 
@@ -28,9 +34,6 @@ use Class::Std;
     # how many times to try connecting to database:
     Readonly my $MAX_CONNECT_TRIES => 3;
 
-    # how many times to try reading the config file into package var:
-    Readonly my $MAX_READ_CONFIG_TRIES => 3;
-
     # see DBI manpage under "prepare_cached".
     # want to use the default option '0', i think, because as sth's
     # are pushed to the array, if they are replaced with '3', 
@@ -40,7 +43,9 @@ use Class::Std;
     Readonly my %pooftypes => (
         varchar     => 'str',   char        => 'str',   text       => 'str',
         tinytext    => 'str',   mediumtext  => 'str',   longtext   => 'str',
-        enum        => 'str',   set         => 'str', 
+        enum        => 'str',   
+        
+        set         => 'arr', 
 
         float       => 'num',   double      => 'num',   decimal    => 'num',
         tinyint     => 'num',   smallint    => 'num',   mediumint  => 'num',
@@ -55,7 +60,6 @@ use Class::Std;
 
     # some package vars that will be set by various methods:
     my $config          = undef;
-    my $config_filename = undef;
     my %table_info      = ( );
     my %table_names     = ( );
 
@@ -94,8 +98,7 @@ use Class::Std;
         # be fascist about context:
         if (!want('BOOL')) {
             Object::POOF::X::DB->throw(
-                message => __PACKAGE__
-                    .   qq{: exists_table() not called in BOOL context.},
+                message => qq{: exists_table() not called in BOOL context.},
             );
         }
 
@@ -129,17 +132,19 @@ use Class::Std;
  
         # this should never be called as a class method, only obj method:
         # this fixes something icky with overload (maybe just w/ Test::Builder)
+        # returning false will throw method not found exception from Class::Std
         return if (!ref $self);  
  
         # if not actively connected, decline the call:
         if (! $self->dbh() ) {
             carp ref $self, qq{ not connected to get info on table '$table'.};
-            return;     # or should it throw an exception?
+            return;     # that throws an exception can't find method.. good nuf
         }
 
         # if table does not exist, return false:
         if (! $self->exists_table( $table )) {
-            return sub { return; }
+            # this is a real 'false' without an exception:
+            return sub { return; };
         }
 
         # if info not cached in package var, try to look it up:
@@ -151,11 +156,165 @@ use Class::Std;
         return sub {
             return $table_info{$table};
         }
- 
+    }
+
+    sub is_lazy {
+        my ($self) = @_;
+        # right now this always returns false, because i don't want
+        # to implement it yet, but if set, then db performs single-field
+        # selects even if 
+        return scalar 0;
+    }
+
+    sub is_greedy {
+        my ($self) = @_;
+        # if greedy and not lazy, selects will do all fields of all tables of
+        # an object instead of just one table. but if lazy, does single-fields.
+        # lazy + greedy = still doesn't do much.
+        return scalar 0;
+    }
+
+
+    # return an array reference containing the names of the primary keys
+    # of the table:
+
+    my $primary_key_fields_of_table_aref_cache = { };
+    my $primary_key_fields_of_table_href_cache = { };
+    sub primary_key_fields_of_table {
+        my ($self, $table) = @_;
+
+        if (!$table || ref $table) {
+            Object::POOF::X::DB->throw(
+                message => q{non-string '$table' to OP:DB primary_key_fields_of_table.},
+            );
+        }
+
+        # if already cached in private package var, return that:
+        if  (   want('REF') eq 'ARRAY'
+            &&  exists $primary_key_fields_of_table_aref_cache->{$table}
+            ) {
+            return $primary_key_fields_of_table_aref_cache->{$table};
+        }
+        elsif
+            (   want('REF') eq 'HASH'
+            &&  exists $primary_key_fields_of_table_href_cache->{$table}
+            ) {
+            return $primary_key_fields_of_table_href_cache->{$table};
+        }
+
+        # Class::Std could return exception from AUTOMETHOD: db not connected.
+        my $table_info; eval { $table_info = $self->$table(); };
+        if (my $e = Exception::Class->caught('Exception::Class')) {
+            $e->rethrow();
+        }
+        elsif ($EVAL_ERROR) {
+            Object::POOF::X::DB->throw(
+                message     
+                    => qq{primary_key_fields_of_table($table)\nEVAL_ERROR: $EVAL_ERROR},
+            );
+        }
+
+        if (!$table_info) {
+            Object::POOF::X::DB->throw(
+                message 
+                    => qq{primary_key_fields_of_table($table): table does not exist.},
+            );
+        }
+
+        my $primary_key_fields_aref 
+            =   $primary_key_fields_of_table_aref_cache->{$table}
+            =   [   grep { $table_info->{info}->{$_}->{key} eq 'pri' } 
+                    @{ $table_info->{fields} }
+                ];
+
+        if    (want('REF') eq 'ARRAY')  {
+            return $primary_key_fields_aref;
+        }
+        elsif (want('LIST')) {
+            return @{$primary_key_fields_aref};
+        } 
+        else {
+            # if it existed, it would have been returned, so it doesn't.
+            my $primary_key_fields_href
+                =   $primary_key_fields_of_table_href_cache->{$table}
+                =   {   map {     # convenient to have table info closeby:
+                            $_ => $table_info->{info}->{$_} 
+                        } 
+                        @{ $primary_key_fields_aref },
+                    };
+#           warn "lame:\n".Dump($primary_key_fields_href);
+#           warn "want REF is   '".want('REF')."'"; 
+#           warn "want ARRAY is '".want('ARRAY')."'";
+#           warn "want HASH is  '".want('HASH')."'";
+#           warn "want LIST is  '".want('LIST')."'";
+#           warn "want SCALAR is'".want('SCALAR')."'";
+#           warn "wantarray is  '".wantarray."'";
+#           warn "howmany is    '".want('COUNT')."'" if defined want('COUNT');
+#           cluck "horta";
+#           Object::POOF::X::DB->throw(
+#               message =>  qq{primary_key_fields_of_table($table):\n}
+#                       .   qq{\tnot HASH/ARRAY (? REF) context.},
+#           );
+            return 
+                (want('REF') || want('SCALAR')) 
+                ?   $primary_key_fields_href 
+                :   %{$primary_key_fields_href};
+        }
+    }
+
+    sub quote {
+        my ($self, $str) = @_;
+        eval { return $self->dbh->quote($str); };
+        if (my $e = Exception::Class->caught('Object::POOF::X')) {
+            $e->rethrow();
+        }
+    }
+
+    my %tbls_w_same_pk_flds_in_class = ( );
+    sub tbls_w_same_pk_flds_in_class {
+        my ($self, $class) = @_;
+
+        $class = ref $class if ref $class;
+
+        if ($tbls_w_same_pk_flds_in_class{$class}) {
+            return scalar 1;
+        }
+        else {
+            my @primary_key_fields = (
+                $self->primary_key_fields_of_table( ${$class->table_ref()} )
+            );
+
+            my @tables_where_pk_flds_dont_match = ( );
+            foreach my $table (@{ $class->more_tables_aref() }) {
+                foreach my $other_pk_field 
+                    ( $self->primary_key_fields_of_table($table) ) {
+                    if  (!scalar    grep { 
+                                        $_ eq $other_pk_field 
+                                    } @primary_key_fields
+                        ) {
+                        push @tables_where_pk_flds_dont_match, 
+                            $table;
+                    }
+                }
+            }
+
+            if (@tables_where_pk_flds_dont_match) {
+                Object::POOF::X::DB->throw(
+                    message 
+                        =>  qq{'$class' primary key fields don't match in\n(}
+                        .   join(q{, }, @tables_where_pk_flds_dont_match)
+                        .   qq{) tables.},
+                );
+            }
+            else {
+                return $tbls_w_same_pk_flds_in_class{$class} 
+                    = scalar 1;
+            }
+        }
     }
 
     sub _lookup_table_info : PRIVATE {
-        my ($self,$table) = @_;
+        my ($self, $table) = @_;
 
         $table_info{$table} = {};
         my $info = $table_info{$table};
@@ -174,6 +333,9 @@ use Class::Std;
             
             $info->{info}->{$fld} = $row;
 
+            # assign the key values... I like lower-case:
+            $info->{info}->{$fld}->{key} = lc $row->{Key};
+
             my ($type, $lim, $unsigned, $zerofill) = $row->{Type} =~ m{
                 \A
                     (\w+)           # type
@@ -201,8 +363,8 @@ use Class::Std;
             # CYA:
             if (!$pooftype) {
                 warn __PACKAGE__
-                    .   qq{->_lookup_table_info() - no valid pooftype.\n}
-                    .   qq{New mysql type '$type'?  Please report a bug.\n};
+                    . qq{->_lookup_table_info() - no valid simple pooftype.\n}
+                    . qq{New mysql type '$type'?  Please report a bug.\n};
             }
             
             # cut up $lim into whatever it might mean for this type:
@@ -210,14 +372,14 @@ use Class::Std;
                 and ( defined $lim ) 
                 and ( $lim =~ m{ \A (\d+) , (\d+) \z }xms )
                 ) {
-                @{ $info->{info}->{$fld} }{'digits','decimals'} = ($1,$2);
+                @{ $info->{info}->{$fld} }{'digits', 'decimals'} = ($1,$2);
             } 
             elsif    
                 (   ( ($type eq 'float') or ($type eq 'double') ) 
                 and ( defined $lim ) 
                 and ( $lim =~ m{ \A (\d+) , (\d+) \z }xms )
                 ) {
-                @{ $info->{info}->{$fld} }{'width','decimals'} = ($1,$2);
+                @{ $info->{info}->{$fld} }{'width', 'decimals'} = ($1,$2);
 
             } 
             elsif 
@@ -225,7 +387,6 @@ use Class::Std;
                 and ( defined $lim ) 
                 ) {
                 $lim =~ s{'}{}g;
-                #warn "lim after is '$lim'\n" if ($d);
                 $info->{info}->{$fld}->{options} = [ split q{,}, $lim ];
             } 
 
@@ -261,9 +422,7 @@ use Class::Std;
             eval {
                 $self->_connect();
             };
-            #warn "in dbh, trying to throw, eval error '$EVAL_ERROR'\n";
-            #warn "which is type '", ref $EVAL_ERROR, "'\n";
-            if (my $e = Exception::Class->caught('Object::POOF::X::DB')) {
+            if (my $e = Exception::Class->caught('Object::POOF::X')) {
                 $e->rethrow();
             }
         }
@@ -293,19 +452,17 @@ use Class::Std;
 
         my $dbh = undef;
         eval { $dbh = $self->dbh };
-        if (my $e = Exception::Class->caught('Object::POOF::X::DB')) {
+        if (my $e = Exception::Class->caught('Object::POOF::X')) {
             # barf it on up
             $e->rethrow();
         }
         
         my $rv = undef;
         eval {
-            #warn "should be evalling something...";
             # if passed a scalar ref, derefence it:
             if  (   (ref $query)
                 and (ref $query eq 'SCALAR') 
                 ) {
-                #warn "trying to dereference scalar ref query.";
                 $rv = $dbh->do(${$query})
             }
             # if passed an Object::POOF::SQL object, 
@@ -313,16 +470,14 @@ use Class::Std;
                 (   (ref $query)
                 and ($query->isa('Object::POOF::SQL') )
                 ) {
-                #warn "trying to stringify Object::POOF::SQL";
                 $rv = $dbh->do( qq{$query} );  # stringify query:
                 # "$query->sql(), $query->attrs(), @{ $query->bind_values() }"
             }
+            # else scalar, so just do it:
             else {
-                #warn "doing scalar query '$query'";
                 $rv = $dbh->do( $query );
             }
         };
-        #if ($EVAL_ERROR) {
         if ($dbh->err()) {
             my $message = undef;
             if (ref $query eq 'SCALAR') {
@@ -344,16 +499,116 @@ use Class::Std;
         return $rv; # return like you'd expect DBI::db->do() to
     }
 
+    sub search_hash {
+        # mostly out of DBI manpage for prepare_cached, w/ enhancements
+        my ($self, $p) = @_;
+
+        my $table   = my $where_href    = my $max_rows  = my $key_field
+            = undef;
+        my @fields  = my @values        = ( );
+
+        eval {
+            $table               = $p->{table};
+            $max_rows            = $p->{max_rows};
+            $key_field           = $p->{key_field};
+            $where_href          = $p->{where_href};
+            @fields              = sort keys %{$where_href};
+            @values              = @{ $where_href }{ @fields };
+        };
+
+        if  (   $EVAL_ERROR
+            ||  !$table     || !$where_href 
+            ||  ref $table  || !ref $where_href eq 'HASH'
+            ||  !scalar @fields
+            ||  (ref $max_rows)
+            ) {
+            my $err = ($EVAL_ERROR) ? qq{$EVAL_ERROR\n} : $EMPTY_STR;
+            Object::POOF::X::DB->throw(
+                message =>  qq{search_hash(): bad params:\n$err}.Dump($p),
+            );
+        }
+
+        # map in the where condition:
+        my $qualifier = $EMPTY_STR;
+        if (scalar @fields) {
+            $qualifier 
+                =   q{ WHERE }
+                .   join(q{ AND }, map { qq{$_ = ?} } @fields);
+        }
+
+        my $sth = undef;
+        # try to get an sth:
+        eval {
+            warn "getting sth for '$table', '$qualifier'";
+            $sth = $self->prepare_cached(qq{
+                SELECT * FROM $table $qualifier
+            });
+        };
+        if (my $e = Exception::Class->caught('Object::POOF::X')) {
+            $e->rethrow();
+        }
+
+        # fail if caller limited max rows returned:
+        if ( ($max_rows) && ($sth->rows() > $max_rows) ) {
+            carp "limited max rows returned";
+            Object::POOF::X::DB->throw(
+                message =>  qq{search_hash(): more rows than $max_rows},
+                params  =>  $p,
+            );
+        }
+
+        my $ref = undef;
+        my $sql = qq{SELECT * FROM $table $qualifier\n};
+        # otherwise return the values:
+        if ($max_rows && $max_rows == 1) {
+            eval {
+                $ref = $self->dbh->selectrow_hashref( 
+                    $sql, $key_field, {}, @values 
+                );
+            };
+        }
+        elsif ($key_field) {
+            eval {
+                $ref = $self->dbh->selectall_hashref( 
+                    $sql, $key_field, {}, @values 
+                );
+            };
+        } 
+        else {
+            eval {
+                $ref = $self->dbh->selectall_arrayref( 
+                    $sql, $key_field, {}, @values 
+                );
+            };
+        }
+
+        if (my $e = Exception::Class->caught('Object::POOF::X')) {
+            $e->rethrow();
+        }
+        elsif ($self->dbh->err()) {
+            Object::POOF::X::DB->throw(
+                message =>  qq{search_hash(): selectall_* failed:},
+                params  =>  $p,
+                errstr  =>  $self->dbh->errstr(),
+            );
+        }
+
+        return $ref;
+    }
+
     sub prepare_cached {
         # works the same way as DBI's "prepare_cached".
         my ($self, $statement, $attr_href) = @_;
+        my $sth = undef;
         eval { 
             # extra arg to prepare() is a bool flag to use prepare_cached
-            return $self->prepare( $statement, $attr_href, 1 ); 
+            $sth = $self->prepare( $statement, $attr_href, 1 ); 
         };
-        if (my $e = Exception::Class->caught('Object::POOF::X::DB')) {
+        if (my $e = Exception::Class->caught('Object::POOF::X')) {
             $e->rethrow();
         }
+
+        return $sth;
     }
 
     sub prepare {
@@ -391,39 +646,34 @@ use Class::Std;
         eval {
             $dbh = $self->dbh();
         };
-        if (my $e = Exception::Class->caught('Object::POOF::X::DB')) {
+        if (my $e = Exception::Class->caught('Object::POOF::X')) {
             $e->rethrow();
         }
 
+        my $sub_prepare_cached = sub {
+            return $dbh->prepare_cached(
+                $statement,
+                $attr_href,
+                $PREPARE_CACHED_IFACTIVE_OPT,
+            );
+        };
+
+        my $sub_prepare = sub {
+            return $dbh->prepare(
+                $statement,
+                $attr_href,
+            );
+        };
+
         my $sth = undef;
-        my $dbifunc = ($cache) 
-            ? sub {
-                return $dbh->prepare_cached(
-                    $statement,
-                    $attr_href,
-                    $PREPARE_CACHED_IFACTIVE_OPT,
-                );
-            }
-            : sub {
-                return $dbh->prepare_cached(
-                    $statement,
-                    $attr_href,
-                );
-            };
+        my $dbifunc = ($cache) ? $sub_prepare_cached : $sub_prepare;
 
-#       warn qq{
-#           statement: '$statement',
-#           attr_href: '$attr_href',
-#           dbifunc:   '$dbifunc',
-#           const:     '$PREPARE_CACHED_IFACTIVE_OPT',
-#       };
-        
         # prepare the statement and get the handle:
         eval { 
             $sth = $dbifunc->();
         };
         if ($EVAL_ERROR) {
-            warn $EVAL_ERROR;
+            carp $EVAL_ERROR;
             Object::POOF::X::DB::STH->throw(
                 message => qq{Could not get sth from DBI $dbifunc.},
                 errstr  => $dbh->errstr(),
@@ -431,8 +681,6 @@ use Class::Std;
         }
 
         # if a valid sth returned, put it in the cleanup stack:
-        my $ident = ident($self);
-        #warn "sth is a '$sth'\n";
         push @{ $sths_of{ ident($self) } }, $sth;
 
         # hmm, when used with prepare_cached, some of these sth
@@ -450,6 +698,14 @@ use Class::Std;
         my $ident = ident($self);
 
         map { $_->finish() } @{ $sths_of{$ident} };
+        foreach my $sth (@{ $sths_of{$ident} }) {
+            eval { $sth->finish() };
+
+            if ($EVAL_ERROR) {
+                carp qq{Object::POOF::DB cannot finish an STH:\n$EVAL_ERROR\n};
+                # but don't throw?
+            }
+        }
         return;
     }
 
@@ -463,7 +719,7 @@ use Class::Std;
                 errstr  => $self->dbh->errstr(),
             );
         }
-        return;
+        return scalar 1;
     }
 
     sub commit {
@@ -476,7 +732,184 @@ use Class::Std;
                 errstr  => $self->dbh->errstr(),
             );
         }
-        return;
+        return scalar 1;
+    }
+
+    sub check_save_pair {
+        my ($self, $object, $field, $value) = @_;
+
+        my $class = ref $object || $object;
+
+        # should only pass non-empty scalars as field
+        # and non-empty ref as object:
+        if  (   !ref $object    ||  ref $field  
+            ||  !$object        ||  !$field     
+            ) {
+            Object::POOF::X::DB->throw(
+                message
+                    =>  qq{Non-scalar or empty field\n}
+                    .   qq{or non-ref or empty object passed\n}
+                    .   qq{to check_save_pair method for $class.\n},
+            );
+        }
+
+        # ref values might actually be allowed, for instance, Datetime obj's
+
+        # hmm, i wonder if this will ever be an issue 
+        # if i need to somehow relax some of the checking...
+        # should i precheck data type, or wait until save?
+        # for now i'm going to do it here.
+
+        # A reason to relax checking might be if you want to
+        # develop with a highly mutable system where you are
+        # changing database fields all the time (due to idiot
+        # project managers making you dance) and you don't want
+        # to keep on fixing the broken exceptions until you're done.
+
+        # I mean after all, data type checking will be done by
+        # DBI anyway, or values will be converted as it does.
+
+        # so maybe, default should do it here, but if relaxed,
+        # it shouldn't do it anywhere at all, unless DBI barfs.
+
+        # so, this will be called by add_save and for each elem
+        # of 'save' param to new(), and will die if $RELAXED_FIELD_CHECKING
+        # is NOT set in the child DB class.
+
+        {   # localize clever unstrictness:
+            no strict 'refs';
+            my $child_db_class = ref $self;
+            if ( ${ "${child_db_class}::RELAXED_FIELD_CHECKING" } ) {
+                return;
+            }
+        }
+
+        # make sure field is in one of this class's tables.
+        # now, if searching for a primary key field, this will throw,
+        # because it can't lookup table of primary key:
+        my @primary_key_fields = ( );
+        eval { @primary_key_fields = $object->primary_key_fields; };
+        if (my $e = Exception::Class->caught('Object::POOF::X')) {
+            $e->rethrow();
+        }
+
+        my $table = undef;
+        if (grep { $_ eq $field } @primary_key_fields) {
+            $table = ${ $object->table_ref() };
+        }
+        else {
+            eval { $table = $object->table_of_field( $field ); };
+            if (my $e = Exception::Class->caught('Object::POOF::X')) {
+                $e->rethrow();
+            }
+        }
+
+        if (!$table) {
+            Object::POOF::X::DB->throw(
+                message 
+                    =>  qq{check_save_pair for $class could not find\n}
+                    .   qq{field $field in its tables:\n\t(}
+                    .   join(q{, }, $object->all_tables() ) .qq{)\n},
+            );
+        }
+
+        eval { $self->_check_field_value_datatype( $table, $field, $value ); };
+        if (my $e = Exception::Class->caught('Object::POOF::X')) {
+            if ($e->isa('Object::POOF::X::DataType')) {
+                $e->{class} = $class;  
+            }
+            $e->rethrow();
+        }
+
+        # else it's okay.
+        return scalar 1;
+    }
+
+    sub _check_field_value_datatype : PRIVATE {  
+        my ($self, $table, $field, $value) = @_;
+
+        if (!$table || !$field || !$value) {
+            Object::POOF::X::DB->throw(
+                message => q{_check_field_value_datatype(): empty params?}
+            );
+        }
+
+        # could also add data type checking here.
+        my $info        = $self->$table->{info};
+        my $type        = $info->{$field}->{type};
+        my $pooftype    = $info->{$field}->{pooftype};
+
+        my $throw = 0;
+
+        given ($type) {
+            last if (!ref $value && $value eq 'NULL');
+
+            when ('float')      { $throw++ if !is_float(    $value  );  }
+            when ('double')     { $throw++ if !is_float(    $value  );  }
+            when ('decimal')    { $throw++ if !is_decimal(  $value  );  }
+            when m{ int \z }xms { $throw++ if !is_int(      $value  );  }
+
+            when ('enum') {
+                if (!grep { $_ eq $value } @{$info->{$field}->{options}}) {
+                    # it could be an index i guess:
+                    if (is_whole($value)) {
+                        if ($value >= scalar @{$info->{$field}->{options}}) {
+                            $throw++;
+                        }
+                    }
+                    else {
+                        $throw++;
+                    }
+                }
+            }
+
+            when ('set') {
+                # make sure value is arrayref:
+                if (ref $value ne 'ARRAY') {
+                    $throw++; 
+                    last;
+                }
+
+                # make sure value array elements are in allowed options:
+                my $options = $info->{$field}->{options};
+                foreach my $elem (@{$value}) {
+                    if (! grep { m{ \A $elem \z }xms } @{$options} ) {
+                        $throw++;
+                        last;
+                    }
+                }
+            }
+
+        }
+
+        # unfortunately the 'dt' pooftype is pretty useless...
+        # maybe i should just *require* that the value is a DateTime object.
+
+
+        given ($pooftype) {
+            last if (!ref $value && $value eq 'NULL');
+
+            when ('str')        { $throw++ if !is_string($value);       }
+
+            when ('dt')         { 
+                if ( ! (ref $value && $value->isa('DateTime') ) ) {
+                    warn "...not a DateTime, so should throw.";
+                    $throw++;
+                }
+            }
+
+        }
+
+        if ($throw) {
+            Object::POOF::X::DataType->throw(
+                message => qq{incorrect value for data type.},
+                table   => $table,
+                field   => $field,
+                value   => $value,
+                field_info => $info->{$field},
+            );
+        }
+        return scalar 1;
     }
 
     sub _ping : PRIVATE {
@@ -503,20 +936,15 @@ use Class::Std;
             # try to return immediately - _connect_attempt() returns dbh
             eval {
                 $self->_connect_attempt();
-                #carp "dbh_of{$ident} is a '", ref $dbh_of{$ident}, "'\n";
             };
 
-            #warn "on connect try $try, eval error is type '", ref $EVAL_ERROR, "'";
-
             # throw up if error caught and it's the last try.
-            if (my $e = Exception::Class->caught('Object::POOF::X::DB') ) {
+            if (my $e = Exception::Class->caught('Object::POOF::X') ) {
                 if ($try == $MAX_CONNECT_TRIES) {
-                    #warn "rethrowing error of type '", ref $EVAL_ERROR, "'";
                     $e->rethrow();
                 }
                 else {
                     # else wait a blip before trying again...
-                    #warn "calling _connect_attempt() again...";
                     usleep( 10 + int( rand(90) ) );
                     next CONNECT;
                 }
@@ -544,7 +972,7 @@ use Class::Std;
 
             eval { $self->_read_config(); };
 
-            if (my $e = Exception::Class->caught('Object::POOF::X::DB') ) {
+            if (my $e = Exception::Class->caught('Object::POOF::X') ) {
                 # undef config again to make it know it didn't work:
                 $config = undef;
 
@@ -587,11 +1015,12 @@ use Class::Std;
         my $read_filename       = undef;
         my $pkg_config_filename = undef;
 
-        my ($baseclass) = $class =~ m{ \A (\w+) :: .* \z }xms;
+        my ($childclass) = $class =~ m{ \A (\w+) :: .* \z }xms;
 
         {   # isolate clever unstrictness:
             no strict 'refs';
-            $pkg_config_filename = ${"${baseclass}::DB::config_filename"};
+            my $child_db_class = ref $self;
+            $pkg_config_filename = ${"${child_db_class}::CONFIG_FILENAME"};
         }
 
         # if filename defined in child package, use it:
@@ -600,7 +1029,7 @@ use Class::Std;
         }
         # else use default:
         else {
-            $read_filename = qq{/etc/poof/$baseclass/config};
+            $read_filename = qq{/etc/poof/$childclass/config};
         }
 
         # try to read in config:
@@ -674,7 +1103,7 @@ This document describes Object::POOF::DB version 0.0.6
  croak $EVAL_ERROR if ($EVAL_ERROR);  
 
  # it will stringify properly, or...
- # if (my $e = Exception::Class->caught('Object::POOF::X::DB') ) {
+ # if (my $e = Exception::Class->caught('Object::POOF::X') ) {
  #     $e->rethrow();
  # }
 
@@ -691,7 +1120,7 @@ This document describes Object::POOF::DB version 0.0.6
  });
 
  eval { $thing->save() };  # Object::POOF::X::Update exception?
- if (my $e = Exception::Class->caught('Object::POOF::X::Save')) {
+ if (my $e = Exception::Class->caught('Object::POOF::X')) {
     $thing->rollback();
     carp $e;
  }
